//
//  HypnosisterView.m
//  Hypnosister
//
//  Created by Benjamin Tan on 20/2/13.
//  Copyright (c) 2013 Benjamin Tan. All rights reserved.
//

#import "HypnosisterView.h"

@implementation HypnosisterView
@synthesize circleColor;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // All HypnosisViews start with a clear background color
        [self setBackgroundColor:[UIColor clearColor]];
        [self setCircleColor:[UIColor lightGrayColor]];
    }
    return self;
}

- (BOOL)canBecomeFirstResponder
{
    return YES;
}

- (void)motionBegan:(UIEventSubtype)motion withEvent:(UIEvent *)event
{
    if (motion == UIEventSubtypeMotionShake) {
        NSLog(@"Device started shaking");
        [self setCircleColor:[UIColor redColor]];
    }
}

- (void)setCircleColor:(UIColor *)clr
{
    circleColor = clr;
    [self setNeedsDisplay];
}

- (void)drawRect:(CGRect)dirtyRect
{
    CGContextRef ctx = UIGraphicsGetCurrentContext();
    CGRect bounds = [self bounds];
    
    // Figure out the center of the bounds rectangle
    CGPoint center;
    center.x = bounds.origin.x + bounds.size.width / 2.0;
    center.y = bounds.origin.y + bounds.size.height / 2.0;
    
    // The radius of the circle should be nearly as b ig as the view
    float maxRadius = hypot(bounds.size.width, bounds.size.height) / 2.0;
    
    // The thickness of the line should be 10 points wide
    CGContextSetLineWidth(ctx, 10);
    
    // Draw concentric circles from the outside in
    
    NSArray *colors = [NSArray arrayWithObjects:[UIColor blueColor],
                                                [UIColor redColor],
                                                [UIColor purpleColor],
                                                [UIColor orangeColor],
                                                [UIColor yellowColor], nil];
    
    for (float currentRadius = maxRadius; currentRadius > 0; currentRadius -= 20) {
        // Add a path to the context
        NSUInteger randomIndex = arc4random() % [colors count];
        [self setCircleColor:[colors objectAtIndex:randomIndex]];
        
        [[self circleColor] setStroke];
        CGContextAddArc(ctx, center.x, center.y, currentRadius, 0.0, M_PI * 2.0, YES);
        CGContextStrokePath(ctx);
    }
    
    // Create a string
    NSString *text = @"You are getting sleepy.";
    
    // Get a dont to draw it in
    UIFont *font = [UIFont boldSystemFontOfSize:28];
    
    CGRect textRect;
    
    // How big is this string when drawn in this font
    textRect.size = [text sizeWithFont:font];
    
    // Let's put that string in the center of the view
    textRect.origin.x = center.x - textRect.size.width / 2.0;
    textRect.origin.y = center.y - textRect.size.height / 2.0;
    
    // Set the fill color of the current context to black
    [[UIColor blackColor] setFill];
  
    // The shadow will move 4 points to the right and 3 points down from the text
    CGSize offset = CGSizeMake(4, 3);
    
    // THe shadow will be dark gray in color
    CGColorRef color = [[UIColor darkGrayColor] CGColor];
    
    CGContextSetShadowWithColor(ctx, offset, 2.0, color);
  
    // Draw the string
    [text drawInRect:textRect withFont:font];
    
    // call a new method to draw the cross-hairs
    [self drawCrossHairs:ctx atPoint:center];
}

-(void)drawCrossHairs:(CGContextRef)ctx atPoint:(CGPoint)center {
    // For the Silver Challenge of this chapter, create a new
    // method to draw the required green crosshairs.
    // Make them "fancy" with a circle around the centre-point, and the
    // centre-point itself not drawn.
    double sizeOfCrossHair=45.0;
    
    CGContextSaveGState(ctx);
    CGContextSetLineWidth(ctx,2);
    
    CGContextBeginPath(ctx);
    CGContextSetRGBStrokeColor(ctx,0.0,1.0,0.0,1.0); // i.e. green
    
    // Horizontal line of the crosshair
    CGContextMoveToPoint(ctx,center.x-sizeOfCrossHair,center.y);
    CGContextAddLineToPoint(ctx,center.x-sizeOfCrossHair/6,center.y);
    CGContextMoveToPoint(ctx,center.x+sizeOfCrossHair/6,center.y);
    CGContextAddLineToPoint(ctx,center.x+sizeOfCrossHair,center.y);
    
    // Vertical line of the crosshair
    CGContextMoveToPoint(ctx,center.x,center.y-sizeOfCrossHair);
    CGContextAddLineToPoint(ctx,center.x,center.y-sizeOfCrossHair/6);
    CGContextMoveToPoint(ctx,center.x,center.y+sizeOfCrossHair/6);
    CGContextAddLineToPoint(ctx,center.x,center.y+sizeOfCrossHair);
    
    // Draw a circle at 80% of the radius
    [self setCircleColor:[UIColor greenColor]];
    [[self circleColor] setStroke];
    
    // First move to the start of the circle, which is at x=1, y=0
    CGContextMoveToPoint(ctx,center.x+0.8*sizeOfCrossHair,center.y);
    CGContextAddArc(ctx, center.x, center.y, 0.8*sizeOfCrossHair, 0.0, M_PI*2.0, YES);
    
    CGContextStrokePath(ctx);
    // Restore the GState
    CGContextRestoreGState(ctx);
}
@end
